@@ -48,7 +48,7 @@ C Client Library for memcached (libmemcached, -lmemcached)
                          memcached_return *error);
   memcached_return 
   memcached_fetch_execute(memcached_st *ptr, 
-                          unsigned int (*callback[])(memcached_st *ptr, memcached_result_st *result, void *context),
+                          memcached_return (*callback[])(memcached_st *ptr, memcached_result_st *result, void *context),
                           void *context,
                           unsigned int number_of_callbacks);
 
@@ -60,7 +60,8 @@ three pointer variables which will give you the state of the returned
 object.  A uint32_t pointer to contain whatever flags you stored with the value,
 a size_t pointer which will be filled with size of of the object, and a
 memcached_return pointer to hold any error. The object will be returned
-upon success and NULL will be returned on failure. 
+upon success and NULL will be returned on failure. Any object returned by
+memcached_get() must be released by the caller application. 
 
 memcached_mget() is used to select multiple keys at once. For multiple key
 operations it is always faster to use this function. This function always
@@ -79,7 +80,10 @@ three pointer variables which will give you the state of the returned
 object.  A uint32_t pointer to contain whatever flags you stored with the value,
 a size_t pointer which will be filled with size of of the object, and a
 memcached_return pointer to hold any error. The object will be returned
-upon success and NULL will be returned on failure. 
+upon success and NULL will be returned on failure. MEMCACHD_END is returned
+by the *error value when all objects that have been found are returned. 
+The final value upon MEMCACHED_END is null. Values returned by
+memcached_fetch() musted be free'ed by the caller.
 
 memcached_fetch_result() is used to return a memcached_result_st(3) structure 
 from a memcached server. The result object is forward compatible with changes
